@@ -12,15 +12,24 @@ Just for internal usage!
 
 =head2 log()
 
+=head2 reload()
+
+=head2 flush()
+
 =head2 errstr()
 
+=head1 PREREQUISITES
+
+    Carp
+    UNIVERSAL
+
 =head1 AUTHOR
 
 Jonny Schulz <jschulz.cpan(at)bloonix.de>.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2007 by Jonny Schulz. All rights reserved.
+Copyright (C) 2007-2008 by Jonny Schulz. All rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -31,11 +40,11 @@ package Log::Handler::Output;
 
 use strict;
 use warnings;
-our $VERSION = '0.00_11';
-our $ERRSTR  = '';
-
 use Carp;
-use Devel::Backtrace;
+use UNIVERSAL;
+
+our $VERSION = "0.08";
+our $ERRSTR  = "";
 
 sub new {
     my ($class, $options, $output) = @_;
@@ -45,13 +54,24 @@ sub new {
 }
 
 sub log {
-    my $self    = shift;
-    my $level   = shift;
-    my $wanted  = {message=>join(' ', @_)||''};
-    my $output  = $self->{output};
-    my $pattern = $self->{pattern};
-    my $message = { };
+    my $self  = shift;
+    my $level = shift;
 
+    if ($self->{filter_caller}) {
+        my $caller = (caller(1+$Log::Handler::CALLER_LEVEL))[0];
+        return 1 if $caller !~ $self->{filter_caller};
+    } elsif ($self->{except_caller}) {
+        my $caller = (caller(1+$Log::Handler::CALLER_LEVEL))[0];
+        return 1 if $caller =~ $self->{except_caller};
+    }
+
+    my $output  = $self->{output};
+    my $message = { };
+    my $wanted  = { message => join(" ", grep defined, @_) };
+
+    # The patterns must be generated for each output. The reason
+    # is that each output can have their own time/date format
+    # and the code which is executed can return another value.
     foreach my $r (@{$self->{wanted_pattern}}) {
         if (ref($r->{code})) {
             $wanted->{$r->{name}} = &{$r->{code}}($self, $level);
@@ -60,6 +80,10 @@ sub log {
         }
     }
 
+    if ($self->{message_pattern}) {
+        &{$self->{message_pattern_code}}($wanted, $message);
+    }
+
     if ($self->{message_layout}) {
         &{$self->{message_layout_code}}($wanted, $message);
     } else {
@@ -67,35 +91,60 @@ sub log {
     }
 
     if ($self->{message_pattern}) {
+        if ($message->{message}) {
+            $wanted->{message} = $message->{message};
+        }
         &{$self->{message_pattern_code}}($wanted, $message);
     }
 
     if ($self->{debug_trace} || $Log::Handler::TRACE) {
         $self->_add_trace($message);
-    } elsif ($self->{newline} && $message->{message} =~ /.\z/) {
-        $message->{message} .= "\n";
     }
 
-    if ($self->{filter}) {
-        $self->_filter_ok($message) or return 1;
+    if ($self->{filter_message}) {
+        $self->_filter_msg($message) or return 1;
     }
 
     if ($self->{prepare_message}) {
-        # each output can have an own prepare function. for this reason
-        # the message is passed as hash and not as a reference.
-        my $prepare_message = $self->_prepare_message(%$message)
-            or return $self->_raise_error($output->errstr);
-        $output->log($prepare_message)
-            or return $self->_raise_error($output->errstr);
-    } else {
-        $output->log($message)
+        eval { &{$self->{prepare_message}}($message) };
+        if ($@) {
+            return $self->_raise_error("prepare_message failed - $@");
+        }
+    }
+
+    if ($self->{newline} && $message->{message} !~ /(?:\015|\012)\z/) {
+        $message->{message} .= "\n";
+    }
+
+    $output->log($message)
+        or return $self->_raise_error($output->errstr);
+
+    return 1;
+}
+
+sub flush {
+    my $self   = shift;
+    my $output = $self->{output};
+
+    if ( UNIVERSAL::can($output, "flush") ) {
+        $output->flush
             or return $self->_raise_error($output->errstr);
     }
 
     return 1;
 }
 
-sub errstr { $ERRSTR }
+sub reload {
+    my ($self, $opts) = @_;
+
+    foreach my $key (keys %$opts) {
+        $self->{$key} = $opts->{$key};
+    }
+}
+
+sub errstr {
+    return $ERRSTR;
+}
 
 #
 # private stuff
@@ -103,39 +152,37 @@ sub errstr { $ERRSTR }
 
 sub _add_trace {
     my ($self, $message) = @_;
+    my @caller = ();
+    my $skip   = $self->{debug_skip};
 
     if ( $message->{message} =~ /.\z/ ) {
         $message->{message} .= "\n";
     }
 
-    my $bt = Devel::Backtrace->new($self->{debug_skip});
+    for (my $i=0; my @c = caller($i); $i++) {
+        my %frame;
+        @frame{qw/package filename line subroutine hasargs wantarray evaltext is_require/} = @c[0..7];
+        push @caller, \%frame;
+    }
 
-    foreach my $point (reverse 0..$bt->points-1) {
-        $message->{message} .= ' ' x 3 . "CALL($point):";
-        my $caller = $bt->point($point);
+    foreach my $i (reverse $skip..$#caller) {
+        $message->{message} .= " " x 3 . "CALL($i):";
+        my $frame = $caller[$i];
         foreach my $key (qw/package filename line subroutine hasargs wantarray evaltext is_require/) {
-            next unless defined $caller->{$key};
+            next unless defined $frame->{$key};
             if ($self->{debug_mode} == 1) { # line mode
-                $message->{message} .= " $key($caller->{$key})";
+                $message->{message} .= " $key($frame->{$key})";
             } elsif ($self->{debug_mode} == 2) { # block mode
-                $message->{message} .= "\n" . ' ' x 6 . sprintf('%-12s', $key) . $caller->{$key};
+                $message->{message} .= "\n" . " " x 6 . sprintf("%-12s", $key) . $frame->{$key};
             }
         }
         $message->{message} .= "\n";
     }
 }
 
-sub _prepare_message {
-    my $self = shift;
-    my $msg  = {@_};
-    my $code = $self->{prepare_message};
-    eval { &$code($msg) };
-    return $@ ? $self->_raise_error($@) : $msg;
-}
-
-sub _filter_ok {
+sub _filter_msg {
     my ($self, $message) = @_;
-    my $filter = $self->{filter};
+    my $filter = $self->{filter_message};
     my $result = $filter->{result};
     my $code   = $filter->{code};
     my $return = ();
@@ -144,7 +191,6 @@ sub _filter_ok {
         $return = &$code($message) || 0;
     } else {
         foreach my $match ( keys %$result ) {
-            #print STDERR "$match: $message->{message} =~ $filter->{$match}\n";
             $result->{$match} =
                 $message->{message} =~ /$filter->{$match}/ || 0;
         }
