@@ -7,7 +7,7 @@ Log::Handler::Output::Screen - Log messages to the screen.
     use Log::Handler::Output::Screen;
 
     my $screen = Log::Handler::Output::Screen->new(
-        log_to => 'STDERR',
+        log_to => "STDERR",
         dump   => 1,
     );
 
@@ -29,7 +29,9 @@ The following options are possible:
 
 =item B<log_to>
 
-Where do you want to log? Possible is: STDOUT and STDERR.
+Where do you want to log? Possible is: STDOUT, STDERR and WARN.
+
+WARN means to call C<warn()>.
 
 The default is STDOUT.
 
@@ -46,7 +48,11 @@ Call C<log()> if you want to log a message to the screen.
 
 Example:
 
-    $screen->log('this message goes to the screen');
+    $screen->log("this message goes to the screen");
+
+=head2 reload()
+
+Reload with a new configuration.
 
 =head2 errstr()
 
@@ -54,7 +60,6 @@ This function returns the last error message.
 
 =head1 PREREQUISITES
 
-    Carp
     Data::Dumper
     Params::Validate
 
@@ -66,21 +71,15 @@ No exports.
 
 Please report all bugs to <jschulz.cpan(at)bloonix.de>.
 
+If you send me a mail then add Log::Handler into the subject.
+
 =head1 AUTHOR
 
 Jonny Schulz <jschulz.cpan(at)bloonix.de>.
 
-=head1 QUESTIONS
-
-Do you have any questions or ideas?
-
-MAIL: <jschulz.cpan(at)bloonix.de>
-
-If you send me a mail then add Log::Handler into the subject.
-
 =head1 COPYRIGHT
 
-Copyright (C) 2007 by Jonny Schulz. All rights reserved.
+Copyright (C) 2007-2008 by Jonny Schulz. All rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -94,8 +93,8 @@ use warnings;
 use Data::Dumper;
 use Params::Validate;
 
-our $VERSION  = '0.00_07';
-our $ERRSTR   = '';
+our $VERSION = "0.0%";
+our $ERRSTR  = "";
 
 sub new {
     my $class   = shift;
@@ -106,23 +105,46 @@ sub new {
 sub log {
     my $self    = shift;
     my $message = @_ > 1 ? {@_} : shift;
+    local $|=1;
 
     if ($self->{dump}) {
-        $message->{message} = Dumper(@_);
+        $message->{message} = Dumper($message);
     }
 
-    if ($self->{log_to} eq 'STDOUT') {
+    if ($self->{log_to} eq "STDOUT") {
         print STDOUT $message->{message}
             or return $self->_raise_error($!);
-    } else {
+    } elsif ($self->{log_to} eq "STDERR") {
         print STDERR $message->{message}
             or return $self->_raise_error($!);
+    } elsif ($self->{log_to} eq "WARN") {
+        warn $message->{message};
     }
 
     return 1;
 }
 
-sub errstr { $ERRSTR }
+sub reload {
+    my $self = shift;
+    my $opts = ();
+
+    eval { $opts = $self->_validate(@_) };
+
+    if ($@) {
+        $ERRSTR = $@;
+        return undef;
+    }
+
+    foreach my $key (keys %$opts) {
+        $self->{$key} = $opts->{$key};
+    }
+
+    return 1;
+}
+
+sub errstr {
+    return $ERRSTR;
+}
 
 #
 # private stuff
@@ -134,8 +156,8 @@ sub _validate {
     my %options = Params::Validate::validate(@_, {
         log_to => {
             type => Params::Validate::SCALAR,
-            regex => qr/^STD(?:ERR|OUT)\z/,
-            default => 'STDOUT',
+            regex => qr/^(?:STDOUT|STDERR|WARN)\z/,
+            default => "STDOUT",
         },
         dump => {
             type => Params::Validate::SCALAR,
@@ -147,4 +169,10 @@ sub _validate {
     return \%options;
 }
 
+sub _raise_error {
+    my $self = shift;
+    $ERRSTR = shift;
+    return undef;
+}
+
 1;
