@@ -7,9 +7,9 @@ Log::Handler::Output::Socket - Send messages to a socket.
     use Log::Handler::Output::Socket;
 
     my $sock = Log::Handler::Output::Socket->new(
-        peeraddr    => '127.0.0.1',
+        peeraddr    => "127.0.0.1",
         peerport    => 44444,
-        proto       => 'tcp',
+        proto       => "tcp",
         timeout     => 10
     );
 
@@ -43,16 +43,14 @@ The protocol you wish to use. Default is TCP.
 
 =item B<timeout>
 
-The timeout to send message. The default is 1.
+The timeout to send message. The default is 5 seconds.
 
-=item B<persistent>
+=item B<persistent> and B<reconnect>
 
-Enable a persistent connection to the server. The default is 1.
+With this option you can enable or disable a persistent connection and
+re-connect if the connection was lost.
 
-=item B<reconnect>
-
-You can enable this if you want to reconnect to the server after the
-connection lost. The default is 1.
+Both options are set to 1 on default.
 
 =item B<dump>
 
@@ -82,7 +80,7 @@ C<IO::Socket::INET> and don't want use C<peeraddr> and C<peerhost>.
 Example:
 
         connect => {
-            PerrAddr  => '127.0.0.1',
+            PerrAddr  => "127.0.0.1",
             PeerPort  => 44444,
             LocalPort => 44445
         }
@@ -97,7 +95,7 @@ Call C<log()> if you want to send a message over the socket.
 
 Example:
 
-    $sock->log('message');
+    $sock->log("message");
 
 =head2 connect()
 
@@ -107,10 +105,21 @@ Connect to the socket.
 
 Disconnect from socket.
 
+=head2 reload()
+
+Reload with a new configuration.
+
 =head2 errstr()
 
 This function returns the last error message.
 
+=head1 PREREQUISITES
+
+    Carp
+    Params::Validate;
+    IO::Socket::INET;
+    Data::Dumper;
+
 =head1 EXPORTS
 
 No exports.
@@ -119,21 +128,15 @@ No exports.
 
 Please report all bugs to <jschulz.cpan(at)bloonix.de>.
 
+If you send me a mail then add Log::Handler into the subject.
+
 =head1 AUTHOR
 
 Jonny Schulz <jschulz.cpan(at)bloonix.de>.
 
-=head1 QUESTIONS
-
-Do you have any questions or ideas?
-
-MAIL: <jschulz.cpan(at)bloonix.de>
-
-If you send me a mail then add Log::Handler into the subject.
-
 =head1 COPYRIGHT
 
-Copyright (C) 2007 by Jonny Schulz. All rights reserved.
+Copyright (C) 2007-2008 by Jonny Schulz. All rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -144,21 +147,24 @@ package Log::Handler::Output::Socket;
 
 use strict;
 use warnings;
+use Carp;
+use Data::Dumper;
 use Params::Validate;
 use IO::Socket::INET;
-use Data::Dumper;
 
-our $VERSION = '0.00_02';
-our $ERRSTR  = '';
+our $VERSION = "0.06";
+our $ERRSTR  = "";
 
 sub new {
-    my $class   = shift;
-    my $options = $class->_validate(@_);
-    my $self    = bless $options, $class;
+    my $class = shift;
+    my $opts  = $class->_validate(@_);
+    my $self  = bless $opts, $class;
+
     if ($self->{persistent}) {
         $self->connect
-            or Carp::croak $self->errstr;
+            or croak $self->errstr;
     }
+
     return $self;
 }
 
@@ -178,16 +184,20 @@ sub log {
             or return undef;
     }
 
-    local $SIG{PIPE} = 'IGNORE';
-    $socket->send($message->{message}) or do {
-        if ($self->{reconnect}) {
+    # If the peer is done then send() croaks
+    eval { $socket->send($message->{message}) };
+
+    if ($@) {
+        if ($self->{persistent} && $self->{reconnect}) {
             $self->connect or return undef;
-            print $socket $message->{message}
-                or return $self->_raise_error("lost connection and reconnect fails: $!");
+            eval { $socket->send($message->{message}) };
+            if ($@) {
+                return $self->_raise_error("something curious happends: $@");
+            }
         } else {
-            return $self->_raise_error("unable to send message: $!");
+            return $self->_raise_error("unable to send message: $@");
         }
-    };
+    }
 
     if (!$self->{persistent}) {
         $self->disconnect;
@@ -218,12 +228,37 @@ sub disconnect {
     $self->{socket} = undef;
 }
 
+sub reload {
+    my $self = shift;
+    my $opts = ();
+
+    eval { $opts = $self->_validate(@_) };
+
+    if ($@) {
+        return $self->_raise_error($@);
+    }
+
+    $self->disconnect;
+
+    foreach my $key (keys %$opts) {
+        $self->{$key} = $opts->{$key};
+    }
+
+    if ($self->{persistent}) {
+        $self->connect
+            or croak $self->errstr;
+    }
+
+    return 1;
+}
+
 sub errstr {
     return $ERRSTR;
 }
 
 sub DESTROY {
     my $self = shift;
+
     if ($self->{socket}) {
         $self->{socket}->close;
     }
@@ -251,7 +286,7 @@ sub _validate {
         },
         proto => {
             type => Params::Validate::SCALAR,
-            default => 'tcp',
+            default => "tcp",
         },
         timeout => {
             type => Params::Validate::SCALAR,
